@@ -1,6 +1,6 @@
 /************************************************************************
 **
-**  Copyright (C) 2021 Kevin B. Hendricks, Stratford, Ontario, Canada
+**  Copyright (C) 2024 Kevin B. Hendricks, Stratford, Ontario, Canada
 **  Copyright (C) 2021 Doug Massay
 **
 **  This file is part of Sigil.
@@ -37,7 +37,9 @@ Controls::Controls(QWidget* parent)
       m_RegexOptionDotAll(false),
       m_RegexOptionMinimalMatch(false),
       m_RegexOptionAutoTokenise(false),
+      m_RegexOptionUnicodeProperty(false),
       m_OptionWrap(false),
+      m_RegexOptionTextOnly(false),
       m_ClearAll(false)
 {
     ui.setupUi(this);
@@ -92,6 +94,8 @@ void Controls::UpdateSearchControls(const QString &text)
         SetRegexOptionDotAll(false);
         SetRegexOptionMinimalMatch(false);
         SetRegexOptionAutoTokenise(false);
+        SetRegexOptionUnicodeProperty(false);
+        SetRegexOptionTextOnly(false);
         return;
     }
 #endif
@@ -124,6 +128,12 @@ void Controls::UpdateSearchControls(const QString &text)
         SetLookWhere("OP");
     } else if (text.contains("NX")) {
         SetLookWhere("NX");
+    } else if (text.contains("SV")) {
+        SetLookWhere("SV");
+    } else if (text.contains("SJ")) {
+        SetLookWhere("SJ");
+    } else if (text.contains("SX")) {
+        SetLookWhere("SX");
     }
 
     // Search Direction
@@ -138,6 +148,8 @@ void Controls::UpdateSearchControls(const QString &text)
     SetRegexOptionDotAll(text.contains("DA"));
     SetRegexOptionMinimalMatch(text.contains("MM"));
     SetRegexOptionAutoTokenise(text.contains("AT"));
+    SetRegexOptionUnicodeProperty(text.contains("UN"));
+    SetRegexOptionTextOnly(text.contains("TO"));
 }
 
 
@@ -155,7 +167,9 @@ QString Controls::GetControlsCode()
     if (m_RegexOptionDotAll) codes.append("DA");
     if (m_RegexOptionMinimalMatch) codes.append("MM");
     if (m_RegexOptionAutoTokenise) codes.append("AT");
+    if (m_RegexOptionUnicodeProperty) codes.append("UN");
     if (m_OptionWrap) codes.append("WR");
+    if (m_RegexOptionTextOnly) codes.append("TO");
     codes.append(GetSearchDirection());
     codes.append(GetLookWhere());
     return codes.join(" ");
@@ -233,6 +247,13 @@ void Controls::SetSearchDirection(QString code)
 }
 
 
+void Controls::SetRegexOptionTextOnly(bool new_state)
+{
+    m_RegexOptionTextOnly = new_state;
+    ui.chkOptionTextOnly->setChecked(new_state);
+}
+
+
 void Controls::SetRegexOptionDotAll(bool new_state)
 {
     m_RegexOptionDotAll = new_state;
@@ -252,6 +273,12 @@ void Controls::SetRegexOptionAutoTokenise(bool new_state)
     ui.chkRegexOptionAutoTokenise->setChecked(new_state);
 }
 
+void Controls::SetRegexOptionUnicodeProperty(bool new_state)
+{
+    m_RegexOptionUnicodeProperty = new_state;
+    ui.chkRegexOptionUnicodeProperty->setChecked(new_state);
+}
+
 void Controls::SetOptionWrap(bool new_state)
 {
     m_OptionWrap = new_state;
@@ -260,7 +287,9 @@ void Controls::SetOptionWrap(bool new_state)
 
 void Controls::DoClearAll()
 {
+    SetRegexOptionTextOnly(false);
     SetOptionWrap(false);
+    SetRegexOptionUnicodeProperty(false);
     SetRegexOptionAutoTokenise(false);
     SetRegexOptionMinimalMatch(false);
     SetRegexOptionDotAll(false);
@@ -275,7 +304,9 @@ void Controls::ExtendUI()
     ui.btClearAll->setDefault(false);
     ui.btClearAll->setAutoDefault(false);
 
+    SetRegexOptionTextOnly(false);
     SetOptionWrap(false);
+    SetRegexOptionUnicodeProperty(false);
     SetRegexOptionAutoTokenise(false);
     SetRegexOptionMinimalMatch(false);
     SetRegexOptionDotAll(false);
@@ -330,6 +361,15 @@ void Controls::ExtendUI()
     ui.cbLookWhere->addItem(tr("NCX File"), "NX");
     look_tooltip += "<dt><b>" + tr("NCX File") + "</b><dd>" + tr("Restrict the find or replace to the NCX file.") + "</dd>";
 
+    ui.cbLookWhere->addItem(tr("Selected SVG Files"), "SV");
+    look_tooltip += "<dt><b>" + tr("Selected SVG Files") + "</b><dd>" + tr("Restrict the find or replace to the SVG files selected in the Book Browser in Code View.") + "</dd>";
+
+    ui.cbLookWhere->addItem(tr("Selected Javascript Files"), "SJ");
+    look_tooltip += "<dt><b>" + tr("Selected JS Files") + "</b><dd>" + tr("Restrict the find or replace to the JS files selected in the Book Browser in Code View.") + "</dd>";
+
+    ui.cbLookWhere->addItem(tr("Selected Misc XML Files"), "SX");
+    look_tooltip += "<dt><b>" + tr("Selected Misc XML Files") + "</b><dd>" + tr("Restrict the find or replace to other XML files selected in the Book Browser under the Misc Folder in Code View.") + "</dd>";
+
     look_tooltip += "</dl>";
     ui.cbLookWhere->setToolTip(look_tooltip);
     ui.cbLookWhere->setCurrentIndex(0);
@@ -353,6 +393,8 @@ void Controls::ConnectSignalsToSlots()
     connect(ui.chkRegexOptionDotAll, SIGNAL(clicked(bool)), this, SLOT(SetRegexOptionDotAll(bool)));
     connect(ui.chkRegexOptionMinimalMatch, SIGNAL(clicked(bool)), this, SLOT(SetRegexOptionMinimalMatch(bool)));
     connect(ui.chkRegexOptionAutoTokenise, SIGNAL(clicked(bool)), this, SLOT(SetRegexOptionAutoTokenise(bool)));
+    connect(ui.chkRegexOptionUnicodeProperty, SIGNAL(clicked(bool)), this, SLOT(SetRegexOptionUnicodeProperty(bool)));
     connect(ui.chkOptionWrap, SIGNAL(clicked(bool)), this, SLOT(SetOptionWrap(bool)));
+    connect(ui.chkOptionTextOnly, SIGNAL(clicked(bool)), this, SLOT(SetRegexOptionTextOnly(bool)));
     connect(ui.btClearAll, SIGNAL(clicked()), this, SLOT(DoClearAll()));
 }
