@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-**  Copyright (C) 2016-2022 Kevin B. Hendricks, Stratford, ON Canada
+**  Copyright (C) 2016-2024 Kevin B. Hendricks, Stratford, ON Canada
 **
 **  This file is part of Sigil.
 **
@@ -40,6 +40,13 @@
 #include "Dialogs/MetaEditorItemDelegate.h"
 #include "Dialogs/MetaEditor.h"
 
+#if QT_VERSION >= QT_VERSION_CHECK(5, 15, 0)
+    #define QT_ENUM_SKIPEMPTYPARTS Qt::SkipEmptyParts
+    #define QT_ENUM_KEEPEMPTYPARTS Qt::KeepEmptyParts
+#else
+    #define QT_ENUM_SKIPEMPTYPARTS QString::SkipEmptyParts
+    #define QT_ENUM_KEEPEMPTYPARTS QString::KeepEmptyParts
+#endif
 
 static const QString SETTINGS_GROUP = "meta_editor";
 static const QString _IN = "  ";
@@ -127,6 +134,14 @@ MetaEditor::MetaEditor(QWidget *parent)
     connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
 
     updateActions();
+    setFocusPolicy(Qt::StrongFocus);
+    view->setFocusPolicy(Qt::StrongFocus);
+    addMetaButton->setFocusPolicy(Qt::StrongFocus);
+    addPropButton->setFocusPolicy(Qt::StrongFocus);
+    delButton->setFocusPolicy(Qt::StrongFocus);
+    tbMoveUp->setFocusPolicy(Qt::StrongFocus);
+    tbMoveDown->setFocusPolicy(Qt::StrongFocus);
+    buttonBox->setFocusPolicy(Qt::StrongFocus);
 }
 
 
@@ -303,7 +318,7 @@ QString MetaEditor::SetNewOPFMetadata(QString& data)
     QString newopfdata = m_opfdata;
     MetadataPieces mdp;
     // Translate from Human Readable Form
-    QStringList dlist = data.split(_RS);
+    QStringList dlist = data.split(_RS, QT_ENUM_SKIPEMPTYPARTS);
     QStringList nlist;
     foreach(QString rc, dlist) {
         if (rc.startsWith(_IN)) {
