@@ -549,7 +549,7 @@ void SearchEditor::Delete()
 void SearchEditor::Reload()
 {
     QMessageBox::StandardButton button_pressed;
-    button_pressed = QMessageBox::warning(this, tr("Sigil"), tr("Are you sure you want to reload all entries?  This will overwrite any unsaved changes."), QMessageBox::Ok | QMessageBox::Cancel);
+    button_pressed = Utility::warning(this, tr("Sigil"), tr("Are you sure you want to reload all entries?  This will overwrite any unsaved changes."), QMessageBox::Ok | QMessageBox::Cancel);
 
     if (button_pressed == QMessageBox::Ok) {
         m_SearchEditorModel->LoadInitialData();
@@ -991,7 +991,7 @@ bool SearchEditor::MaybeSaveDialogSaysProceed(bool is_forced)
         QMessageBox::StandardButton button_pressed;
         QMessageBox::StandardButtons buttons = is_forced ? QMessageBox::Save | QMessageBox::Discard
                                                : QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel;
-        button_pressed = QMessageBox::warning(this,
+        button_pressed = Utility::warning(this,
                                               tr("Sigil: Saved Searches"),
                                               tr("The Search entries may have been modified.\n"
                                                       "Do you want to save your changes?"),
