@@ -1,6 +1,6 @@
 /************************************************************************
 **
-**  Copyright (C) 2015-2023 Kevin B. Hendricks  Stratford, ON Canada
+**  Copyright (C) 2015-2024 Kevin B. Hendricks  Stratford, ON Canada
 **  Copyright (C) 2013      John Schember <john@nachtimwald.com>
 **  Copyright (C) 2009-2011 Strahinja Markovic  <strahinja.markovic@gmail.com>
 **
@@ -57,6 +57,8 @@
     #define QT_ENUM_KEEPEMPTYPARTS QString::KeepEmptyParts
 #endif
 
+static const QString MEDIA_PLAYBACK_ACTIVE_CLASS = "media:playback-active-class";
+static const QString MEDIA_ACTIVE_CLASS          = "media:active-class";
 static const QString SIGIL_VERSION_META_NAME  = "Sigil version";
 static const QString OPF_XML_NAMESPACE        = "http://www.idpf.org/2007/opf";
 static const QString FALLBACK_MIMETYPE        = "text/plain";
@@ -561,6 +563,30 @@ QStringList OPFResource::GetSpineOrderBookPaths() const
     return book_paths_in_reading_order;
 }
 
+
+QStringList OPFResource::GetMediaOverlayActiveClassSelectors() const
+{
+    QReadLocker locker(&GetLock());
+    QStringList activeclassselectors;
+    QString source = CleanSource::ProcessXML(GetText(),"application/oebps-package+xml");
+    OPFParser p;
+    p.parse(source);
+    for (int i=0; i < p.m_metadata.count(); ++i) {
+        if (p.m_metadata.at(i).m_name == "meta") {
+            MetaEntry me = p.m_metadata.at(i);
+            QString propval = me.m_atts.value("property", "");
+            if (propval == MEDIA_ACTIVE_CLASS) {
+                activeclassselectors << "." + me.m_content;
+            }
+            if (propval == MEDIA_PLAYBACK_ACTIVE_CLASS) {
+                activeclassselectors << "." + me.m_content;
+            }
+        }
+    }
+    return activeclassselectors;
+}
+
+
 QString OPFResource::GetPrimaryBookTitle() const
 {
     QString title = "";
@@ -726,7 +752,6 @@ void OPFResource::AddCoverMetaForImage(const Resource *resource, OPFParser &p)
     }
 }
 
-
 void OPFResource::BulkRemoveResources(const QList<Resource *>resources)
 {
     QWriteLocker locker(&GetLock());
@@ -1200,7 +1225,8 @@ void OPFResource::ResourceRenamed(const Resource *resource, QString old_full_pat
 void OPFResource::ResourceMoved(const Resource *resource, QString old_full_path)
 {
     QWriteLocker locker(&GetLock());
-    QString source = CleanSource::ProcessXML(GetText(),"application/oebps-package+xml");
+    // QString source = CleanSource::ProcessXML(GetText(),"application/oebps-package+xml");
+    QString source = GetText();
     OPFParser p;
     p.parse(source);
     // first convert old_full_path to old_bkpath
@@ -1224,6 +1250,60 @@ void OPFResource::ResourceMoved(const Resource *resource, QString old_full_path)
 }
 
 
+void OPFResource::BulkResourcesMoved(const QHash<QString, Resource *> movedDict)
+{
+    QWriteLocker locker(&GetLock());
+    QString opf_start_dir = Utility::startingDir(GetRelativePath());
+    QString source = CleanSource::ProcessXML(GetText(),"application/oebps-package+xml");
+    OPFParser p;
+    p.parse(source);
+
+    // a move should not impact the id so leave the old unique manifest id unchanged
+    for (int i=0; i < p.m_manifest.count(); ++i) {
+        QString href = p.m_manifest.at(i).m_href;
+        QString bookpath = Utility::buildBookPath(href, opf_start_dir);
+        if (movedDict.contains(bookpath)) {
+            Resource * resource = movedDict[bookpath];
+            ManifestEntry me = p.m_manifest.at(i);
+            QString old_me_href = me.m_href;
+            me.m_href = Utility::URLEncodePath(GetRelativePathToResource(resource));
+            p.m_idpos[me.m_id] = i;
+            p.m_hrefpos.remove(old_me_href);
+            p.m_hrefpos[me.m_href] = i;
+            p.m_manifest.replace(i, me);
+        }
+    }
+    UpdateText(p);
+}
+
+
+void OPFResource::BulkResourcesRenamed(const QHash<QString, Resource *> renamedDict)
+{
+    QWriteLocker locker(&GetLock());
+    QString opf_start_dir = Utility::startingDir(GetRelativePath());
+    QString source = CleanSource::ProcessXML(GetText(),"application/oebps-package+xml");
+    OPFParser p;
+    p.parse(source);
+
+    // a rename should not impact the id so leave the old unique manifest id unchanged
+    for (int i=0; i < p.m_manifest.count(); ++i) {
+        QString href = p.m_manifest.at(i).m_href;
+        QString bookpath = Utility::buildBookPath(href, opf_start_dir);
+        if (renamedDict.contains(bookpath)) {
+            Resource * resource = renamedDict[bookpath];
+            ManifestEntry me = p.m_manifest.at(i);
+            QString old_me_href = me.m_href;
+            me.m_href = Utility::URLEncodePath(GetRelativePathToResource(resource));
+            p.m_idpos[me.m_id] = i;
+            p.m_hrefpos.remove(old_me_href);
+            p.m_hrefpos[me.m_href] = i;
+            p.m_manifest.replace(i, me);
+        }
+    }
+    UpdateText(p);
+}
+
+
 int OPFResource::GetCoverMeta(const OPFParser& p) const
 {
     for (int i = 0; i < p.m_metadata.count(); ++i) {
