@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 # vim:ts=4:sw=4:softtabstop=4:smarttab:expandtab
 
-# Copyright (c) 2014 Kevin B. Hendricks, John Schember, and Doug Massay
+# Copyright (c) 2014-2023 Kevin B. Hendricks, John Schember, and Doug Massay
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without modification,
@@ -51,8 +51,10 @@ def xmldecode(data):
     newdata = newdata.replace('&amp;', '&')
     return newdata
 
+WHITESPACE_CHARS = (' ', '\n', '\r', '\t')
+
 SPECIAL_HANDLING_TAGS = OrderedDict([
-    ('?xml', ('xmlheader', -1)), 
+    ('?xml', ('xmlheader', -1)),
     ('!--',  ('comment', -3)),
     ('!DOCTYPE', ('doctype', -1))
     ])
@@ -215,8 +217,8 @@ class Opf_Parser(object):
             p += 1
             while p < n and s[p:p+1] == ' ' : p += 1
         b = p
-        # handle comment special case as there may be no spaces to 
-        # delimit name begin or end 
+        # handle comment special case as there may be no spaces to
+        # delimit name begin or end
         if s[b:].startswith('!--'):
             p = b+3
             tname = '!--'
@@ -237,13 +239,13 @@ class Opf_Parser(object):
         if ttype is None:
             # parse any attributes of begin or single tags
             while s.find('=',p) != -1 :
-                while p < n and s[p:p+1] == ' ' : p += 1
+                while p < n and s[p:p+1] in WHITESPACE_CHARS : p += 1
                 b = p
                 while p < n and s[p:p+1] != '=' : p += 1
                 aname = s[b:p].lower()
-                aname = aname.rstrip(' ')
+                aname = aname.rstrip(' \n\r\t')
                 p += 1
-                while p < n and s[p:p+1] == ' ' : p += 1
+                while p < n and s[p:p+1] in WHITESPACE_CHARS: p += 1
                 if s[p:p+1] in ('"', "'") :
                     qt = s[p:p+1]
                     p = p + 1
@@ -311,7 +313,7 @@ class Opf_Parser(object):
         for (gtype, gtitle,  ghref) in self.guide:
             guide.append((gtype, gtitle, ghref))
         return guide
-    
+
     def get_bindings(self):
         bindings = []
         for (mtype, handler) in self.bindings:
@@ -328,7 +330,7 @@ class Opf_Parser(object):
             xmlres.append(' %s="%s"' % (key, val))
         xmlres.append('>\n')
         return "".join(xmlres)
-                          
+
     def convert_metadata_attr_to_xml(self):
         xmlres = []
         attr = self.metadata_attr
@@ -339,7 +341,7 @@ class Opf_Parser(object):
             xmlres.append(' %s="%s"' % (key, val))
         xmlres.append('>\n')
         return "".join(xmlres)
-        
+
     def convert_metadata_entries_to_xml(self):
         xmlres = []
         for (mname, mcontent, attr) in self.metadata:
@@ -358,7 +360,7 @@ class Opf_Parser(object):
     def convert_manifest_entries_to_xml(self):
         xmlres = []
         for (id, href, mtype, attr) in self.manifest:
-            # all hrefs should be kept in quoted (encoded) form 
+            # all hrefs should be kept in quoted (encoded) form
             xmlres.append('    <item id="%s" href="%s" media-type="%s"' % (id, href, mtype))
             for key in attr:
                 val= attr[key]
@@ -390,14 +392,14 @@ class Opf_Parser(object):
                     xmlres.append(' %s="%s"' % (key, val))
             xmlres.append('/>\n')
         return "".join(xmlres)
-    
+
     def convert_guide_entries_to_xml(self):
         xmlres=[]
         for (gtype, gtitle, ghref) in self.guide:
             # all hrefs should already be in quoted (encoded) form
             xmlres.append('    <reference type="%s" title="%s" href="%s"/>\n' % (gtype, gtitle, ghref))
         return "".join(xmlres)
-                          
+
     def convert_binding_entries_to_xml(self):
         xmlres=[]
         for (mtype, handler) in self.bindings:
@@ -422,9 +424,9 @@ class Opf_Parser(object):
             xmlres.append('  <guide>\n')
             xmlres.append(self.convert_guide_entries_to_xml())
             xmlres.append('  </guide>\n')
-        if len(self.bindings) > 0 and opver.startswith('3'):
+        if len(self.bindings) > 0 and opfver.startswith('3'):
             xmlres.append('  <bindings>\n')
-            xmlres.append(self.guide.convert_binding_entries_to_xml())
+            xmlres.append(self.convert_binding_entries_to_xml())
             xmlres.append('  </bindings>\n')
         xmlres.append('</package>\n')
         return "".join(xmlres)
@@ -457,10 +459,7 @@ def main():
     # print(op.get_guide())
     # print(op.get_bindings())
     print(op.rebuild_opfxml())
-    return 0    
-
-
-
+    return 0
 
 if __name__ == '__main__':
     sys.exit(main())
