@@ -1,10 +1,10 @@
 /************************************************************************
 **
-**  Copyright (C) 2016-2023  Kevin B Hendricks, Stratford, Ontario, Canada
-**  Copyright (C) 2012       John Schember <john@nachtimwald.com>
-**  Copyright (C) 2012       Dave Heiland
-**  Copyright (C) 2012       Grant Drake
-**  Copyright (C) 2009, 2010, 2011  Strahinja Markovic  <strahinja.markovic@gmail.com>
+**  Copyright (C) 2016-2024 Kevin B Hendricks, Stratford, Ontario, Canada
+**  Copyright (C) 2012      John Schember <john@nachtimwald.com>
+**  Copyright (C) 2012      Dave Heiland
+**  Copyright (C) 2012      Grant Drake
+**  Copyright (C) 2009-2011 Strahinja Markovic  <strahinja.markovic@gmail.com>
 **
 **  This file is part of Sigil.
 **
@@ -217,6 +217,14 @@ void FlowTab::CodeView()
     QApplication::restoreOverrideCursor();
 }
 
+void FlowTab::ThemeChangeRefresh()
+{
+    if (m_wCodeView) {
+        m_wCodeView->SetAppearance();
+        m_wCodeView->RehighlightDocument();
+    }
+}
+
 void FlowTab::LoadTabContent()
 {
     // In CV, this call has nothing to do, as the resource is connected to QTextDocument.
