@@ -1,6 +1,6 @@
 /************************************************************************
 **
-**  Copyright (C) 2015-2023 Kevin B. Hendricks, Stratford Ontario Canada
+**  Copyright (C) 2015-2024 Kevin B. Hendricks, Stratford Ontario Canada
 **  Copyright (C) 2009-2011  Strahinja Markovic  <strahinja.markovic@gmail.com>
 **
 **  This file is part of Sigil.
@@ -31,6 +31,7 @@
 #include "ResourceObjects/HTMLResource.h"
 #include "ResourceObjects/ImageResource.h"
 #include "ResourceObjects/MiscTextResource.h"
+#include "ResourceObjects/XMLResource.h"
 #include "ResourceObjects/SVGResource.h"
 #include "ResourceObjects/PdfResource.h"
 #include "Tabs/PdfTab.h"
@@ -40,6 +41,7 @@
 #include "Tabs/FlowTab.h"
 #include "Tabs/ImageTab.h"
 #include "Tabs/MiscTextTab.h"
+#include "Tabs/XMLTab.h"
 #include "Tabs/SVGTab.h"
 #include "Tabs/NCXTab.h"
 #include "Tabs/OPFTab.h"
@@ -191,14 +193,6 @@ void TabManager::ReloadTabDataForResources(const QList<Resource *> &resources)
 
 void TabManager::ReopenTabs()
 {
-#ifdef Q_OS_MAC
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
-    // for some reason using Qt6 the Tab Manager is not inheritiing the dark/light color change
-    // from its parents when the user changes them and then it overrides the CodeView palette change
-    QPalette app_pal = QApplication::palette();
-    setPalette(app_pal);
-#endif
-#endif
     ContentTab *currentTab = GetCurrentContentTab();
     QList<Resource *> resources = GetTabResources();
     foreach(Resource *resource, resources) {
@@ -208,6 +202,15 @@ void TabManager::ReopenTabs()
     OpenResource(currentTab->GetLoadedResource(), -1, -1, QString());
 }
 
+void TabManager::PerformThemeChangeRefresh()
+{
+    for (int i = 0; i < count(); ++i) {
+        ContentTab *tab = qobject_cast<ContentTab *>(widget(i));
+        if (tab) {
+            tab->ThemeChangeRefresh();
+        }
+    }
+}
 
 void TabManager::SaveTabData()
 {
